#include <stdio.h>
#include <type.h>
#include <board.h>
#include <codec.h>
#include <mcbsp54.h>


int codewheel_var = 0;
HANDLE hHandset;
s16 data[];


/**************************************************************************
************************ main function ************************************
1. set up interrupts
2. set up tlc320AD50 (initailise MsCbs serial port, set sampling rate etc)
3. set up dsp to host communications (ie RTDX)
4. wait for events
5. perform braille conversions on host and display output
***************************************************************************
***************************************************************************/
int main(void)
{

//**************1. set upt interrupts*************

//**************2. set upt tlc320AD50*************
// Open Handset Codec 
    hHandset = codec_open(HANDSET_CODEC);               // Acquire handle to codec 

    // Set codec parameters 
    codec_adc_mode(hHandset, CODEC_ADC_15BIT);          // ADC in 15-bit mode 
    codec_ain_gain(hHandset, CODEC_AIN_6dB);            // 6dB gain on analog input to ADC 
    codec_sample_rate(hHandset,SR_22000);               // 22KHz sampling rate 

/**************3. set up dsp to host communications (ie RTDX)*************
#include <rtdx.h>       // defines RTDX target API calls            
#include "target.h"     // defines TARGET_INITIALIZE()              
#include <stdio.h>      // C_I/O                                    

// These are the values we are going to send to the host            
#define VALUES_TO_SEND {1,2,3,4,5,6,7,8,9,10}

// declare a global output channel                                  
RTDX_CreateOutputChannel( ochan );
 
void main()
{
        int arraydata[] = VALUES_TO_SEND;
        int status;

        TARGET_INITIALIZE();

        // enable the output channel                                
        RTDX_enableOutput( &ochan );

        // to send the data to the host
status = RTDX_write( &ochan, arraydata, sizeof(arraydata) ); */

//**************4. wait for events*************
asm("Here: B Here ; Stays here indefinitely...");

//**************5. perform braille conversions on host and display output*************
/*	-for rtdx, using ms-dos type command eg. c:ti/tutorial/dsk5402/sect_1/less_2/S1L2.exe
	 is good example of sending an array of integers to host from target
*/

}


/************************************************************************************
******************** SI_interrupt ***************************************************
1. record the CCD output (57 samples after SI pulse)
2. determine dots within image slice of 57 samples
3. store then correlate with previous or latter slices constituting a braille cell
4. determine braille cell
5. send to host(use rtdx, does not require dsp/bios)
*************************************************************************************
*************************************************************************************/
void int0_si_interrupt()
{
asm("	RSBX INTM");		//disable all interrupts from interrupting this routine

int sample_var = 0;				//sample number within an image slice

if (codewheel_var == 1)    {

//**************1. record the CCD output (57 samples after SI pulse)*********	
for (sample_var = 0; sample_var != 57; ++sample_var)  { 
		   // Wait for sample from handset 
       while (!MCBSP_RRDY(HANDSET_CODEC)) {};

       // Read sample from handset codec 
       data[sample_var] = *(volatile u16*)DRR1_ADDR(HANDSET_CODEC);
			}

/***********2. determine dots within image slice of 57 samples***************
  a. use at least 3 image slices to detemine dots in a "slice', since ccd
 	 may not be perfectly horizontal
  b. for every 5 or so ccd samples (breaking image slice into about 10 parts), check
  	 to see if a dot exists. Since the distance between dots is precisely known and
  	 constant from the first dot found, corresponding area's where the dots could be 
  	 found are checked. Therefore the actual place of the first dot can vary and proper
  	 braille cell type can still be determined*/

/*****3. store then correlate with previous or latter slices constituting a braille cell**
	- Since initially the ccd could be placed anywhere on the braille line, the first image 
	slice will only be used if the next image slice contituting dots is inside the width
	of a braille cell (determined by counting the codewheel interrupts) otherwise it is
	discarded (as being the the second row of a braille cell)
	- 
/*****************4. determine braille cell**********************************/
/****************5. send to host(use rtdx, does not require dsp/bios)*******/
			
codewheel_var = 0;		//reset codewheel_var, is set every time X_INT0 occurs, ie every 5.3 mS
}


/**************************************************************************************
********************** codewheel_interrupt ********************************************
The SI Pulse provides an interrupt to X_INT0 every 5.3 mS, 
the DSP will then only process the CCD image slice from the AIC if 
a X_INT1 event (indicating a codewheel trigger signal) 
occurred in the last 5.3 mS AND XF equals zero (indicating forward motion)
***************************************************************************************/
void int1_codewheel_interrrupt()
{
if (codewheel_var == 1) {
	brd_led_toggle(BRD_LED0); /*"going too fast", codewheel has sent two interrupts before
	 							ccd/dsp has had time to process first*/
	  }
			
if ((BIO == 0) && (codewheel_var == 0) {  //ie scanner is moving slowly forward
	codewheel_var = 1;		  	//codewheel trigger interrupt has occurred
	}
}
