/*********************************************************************************
CCD Output - braille project C5402 target software
Andrew Paquale 09913770
26/7/2002
**********************************************************************************/

#include <type.h>
#include <board.h>
#include <codec.h>
#include <mcbsp54.h>
#include <stdio.h>
#include "target.h"
#include "intr.h"

void codewheel_interrupt0();
void SI_interrupt2();

/*****************************************************************************/
/* Global Variables                                                          */
/*****************************************************************************/
HANDLE hHandset;
s16 data;
int data_array[60];
static short codewheel_var = 0;
int i = 0, newline_var = 0;
int reference_amplitude, reference_amplitudeL, reference_amplitudeR;		
int sam31, sam32, sam33, sam34, sam35, sam36, sam37;	//sample number, fuzzy dot variable
int sam38, sam39, sam40, sam41, sam42, sam43;	//sample number, fuzzy dot variable
int sam44, sam45, sam46, sam47, sam48;	//sample number, fuzzy dot variable
int dot1, dot2, dot3, dot1_value, dot2_value, dot3_value;
int slice_new, slice_old1, slice_old2;
int braille_cell, slice_goodnew, slice_good1, slice_good2, slice_good3, slice_good4;
int slice_good5, slice_good6, slice_good7, slice_good8, slice_good9, slice_good10;
char temp[80];
int a, b, c;


extern int bio_test()
{
asm("_test_bio:");
asm("	LD #0, A ;Set return value to false ");
asm("	BC bio_low, NBIO ;NBIO = high");
asm("	;BIO = 0 => moving forward");
asm("	NOP");
asm("	NOP ");
asm("	LD #1, A ;Set return value to true ");
asm("	RET ;Card present, return normally");
asm("bio_low: ;bio is low ");
asm("	RET ;exit");
}

void setup_interrupt()
{
	asm(" SSBX INTM");	// Globally disable interrupts
	asm(" STM #5,1h");	// Clear any previous interrupt in IFR
	asm(" STM #4,0h");	// Enable the int2 interrupt, disable codewheel(int0) in IMR
	asm(" RSBX INTM");	// Globally enable interrupts
}
/*****************************************************************************************
--------------------------------------- main function ------------------------------------
1. set up interrupts
2. set up tlc320AD50 (initailise MsCbs serial port, set sampling rate etc)
3. wait for events
*****************************************************************************************/

void main()
{
    
    if (brd_init(100))
        return;

//-------------------------1. set up interrupts-------------------------------------------	
	
	INTR_INIT								//initailise pointer to vector table
	INTR_HOOK( 16,codewheel_interrupt0)		//put ISR into vector table
	INTR_HOOK( 18,SI_interrupt2)			//put ISR into vector table
	
	// Setup the interrupt handler
	setup_interrupt();
	port0 = 0x80;			//sets CNTL1 register, enable int2
	
//-------------------------2. Setup the ADC ---------------------------------------------	
	
    /* Open Handset Codec */
    hHandset = codec_open(HANDSET_CODEC);               // Acquire handle to codec 

    // Set codec parameters 
    codec_adc_mode(hHandset, CODEC_ADC_15BIT);          // ADC in 15-bit mod
    codec_ain_gain(hHandset, CODEC_AIN_0dB);            // 0dB gain on analog input to ADC 
    codec_sample_rate(hHandset,SR_16000);               // 16KHz sampling rate 
	i = 0;

while (1);
    /* Polling and digital loopback */
    
	return;
}


/**************************************************************************************
---------------------- codewheel_interrupt --------------------------------------------
The SI Pulse provides an interrupt to X_INT1 every 5.3 mS, 
the DSP will only process the CCD image slice from the AIC if 
a X_INT0 event (indicating a codewheel trigger signal) 
occurred in the last 5.3 mS AND XF equals zero (indicating forward motion)
***************************************************************************************/
interrupt void codewheel_interrupt0()
{
asm(" SSBX INTM");		// Disable interrupts


codewheel_var = bio_test();		//test to see if codewheel is moving forward
if (codewheel_var == 1) { 		//enable the SI interrupt
	brd_led_enable(BRD_LED0); 
	brd_led_enable(BRD_LED1);
	asm("	SSBX 1, 13");
	asm("	STM #5,IFR");	// Clear the interrupt0 so may be used again (IFR)
	asm(" 	STM #4,0h");		//but disable it until SI interrupt service routine 
	}							//completes,and enable SI interrupt 1 (IMR)
	
if (codewheel_var == 0)	{ 		//keep waiting, dont enable SI interrupt
	brd_led_disable(BRD_LED0); 
	brd_led_disable(BRD_LED1);
	brd_led_disable(BRD_LED2);
	}
asm(" RSBX INTM");		//globally enable interrupts

}

/************************************************************************************
-------------------- SI_interrupt ---------------------------------------------------
1. record the CCD output (57 samples after SI pulse)
2. determine dots within image slice of 57 samples
3. store then correlate with previous or latter slices constituting a braille cell
4. determine braille cell
5. send to host(use rtdx, does not require dsp/bios)
*************************************************************************************/
interrupt void SI_interrupt2()
{

asm(" SSBX INTM");		// Disable interrupts

//----------------------- 1. record the CCD output (57 samples after SI pulse)--------------	
while (i < 59)
    {
       /* Wait for sample from handset */
       while (!MCBSP_RRDY(HANDSET_CODEC)) {};

       /* Read sample from and write back to handset codec */
       data = *(volatile u16*)DRR1_ADDR(HANDSET_CODEC);
       data_array[i] = data;
	   ++i;
    }

/*--------------------2. determine dots within image slice of 57 samples--------------------
  a. use at least 3 image slices to detemine dots in a "slice', since ccd
 	 may not be perfectly horizontal
  b. apply fuzzy logic fundamentals to determine dots within sample array*/
/* i = 20;	// first 20 samples are out of lens view
 while (i < 58) {
 	if (newline_var == 20) {printf("\n"); newline_var = 0;}	//prevent output window horizontal overflow
	printf(" %d",  data_array[i]);
	++i;
	++newline_var;
	}
*/	
reference_amplitudeL = ((data_array[27] / 3) + (data_array[28] / 3) + (data_array[29] / 3));	//divide by 3 to prevent overflow
reference_amplitudeR = ((data_array[50] / 3) + (data_array[51] / 3) + (data_array[52] / 3));	//divide by 3 to prevent overflow
if (reference_amplitudeL >= reference_amplitudeR)		//try find reference amplitude least
	{reference_amplitude = reference_amplitudeL;}		//affected by dot shadows
else { reference_amplitude = reference_amplitudeR;}
	
sam31 = (reference_amplitude - data_array[31]);	//since the sample is subbed from the reference
sam32 = (reference_amplitude - data_array[32]); //it does no matter if the dot is not there
sam33 = (reference_amplitude - data_array[33]); //because the result will be almost zero and thus
sam34 = (reference_amplitude - data_array[34]); //does not affect the fuzzy determined result	
sam35 = (reference_amplitude - data_array[35]); // (the dot shadow due to its limited size is only
sam36 = (reference_amplitude - data_array[36]); // ever going to affect a few of the samples)
sam37 = (reference_amplitude - data_array[37]);		
sam37 = (reference_amplitude - data_array[38]);
sam39 = (reference_amplitude - data_array[39]);
sam40 = (reference_amplitude - data_array[40]); 	
sam41 = (reference_amplitude - data_array[41]);  	
sam42 = (reference_amplitude - data_array[42]);  	
sam43 = (reference_amplitude - data_array[43]);  		
sam43 = (reference_amplitude - data_array[44]);
sam45 = (reference_amplitude - data_array[45]); 
sam46 = (reference_amplitude - data_array[46]);	
sam47 = (reference_amplitude - data_array[47]);  	
sam48 = (reference_amplitude - data_array[48]); 	
  		
//determine using fuzzy priniciples if a dot exists 
dot1_value = ((sam31 /10)+(sam32 /10)+(sam33 /10)+(sam34 /10)+(sam35 /10)+(sam36 /20)+(sam37 /50));		//divide by 5 to prevent overflow
dot2_value = ((sam37 /40)+(sam38 /20)+(sam39 /10)+(sam40 /10)+(sam41 /10)+(sam42 /20)+(sam43 /40));		//divide by 5 to prevent overflow
dot3_value = ((sam42 /50)+(sam43 /20)+(sam44 /10)+(sam45 /10)+(sam46 /10)+(sam47 /10)+(sam48 /30));		//divide by 5 to prevent overflow
if (dot1_value > (reference_amplitude / 5)) { dot1 = 1;}	
else { dot1 = 0; }
if (dot2_value > (reference_amplitude / 5)) { dot2 = 1;}
else { dot2 = 0;}
if (dot3_value > (reference_amplitude / 5)) { dot3 = 1;}		
else { dot3 = 0;}

printf("\n %d%d%d. ", dot1, dot2, dot3);

/*-----3. store then correlate with previous or latter slices constituting a braille cell---
	- Since initially the ccd could be placed anywhere on the braille line, the first image 
	slice will only be used if the next image slice contituting dots is inside the width
	of a braille cell (determined by counting the codewheel interrupts) otherwise it is
	discarded (as being the the second row of a braille cell)
	- need to find greatest image slice over 2,3 since single dot shadows appear over a number 
	slices
	- to find corresponding braille slice to fit with new slice to make a braille cell,need to 
	take into account 
	*/

//get highest value image over range of slices affected by dot shadow for correct braille cell
slice_new = (dot1 * 0x04) + (dot2 * 0x02) + (dot3 * 0x01);  //ie, 110=0x06, 011=0x03, 001=0x01 etc 
if ((slice_old1 >= slice_old2) && (slice_old1 > slice_new)) { slice_goodnew = slice_old1;}
else {slice_goodnew = 0;}
printf ("  slice = %d.", slice_goodnew);

slice_old2 = slice_old1;		//refresh the reference slices
slice_old1 = slice_new;

//correlate dots on right hand side of braille cell with dots on left hand side of braille cell
if ((slice_goodnew != 0) && (slice_good5 != 0)) { 		//for double sided braille cells
	braille_cell = slice_goodnew + (slice_good5 *8);	//only good_slice 3,4, or 5 will != 0
	printf("  braille cell = %d", braille_cell);}
if ((slice_goodnew != 0) && (slice_good4 != 0)) { 		
	braille_cell = slice_goodnew + (slice_good4 *8);
	printf("  braille cell = %d", braille_cell);}
if ((slice_goodnew != 0) && (slice_good3 != 0)) { 
	braille_cell = slice_goodnew + (slice_good3 *8);
	printf("  braille cell = %d", braille_cell);}

//for single sided braille cells display, check it really isn't a double sided cell	
if ((slice_good5 != 0) && (slice_goodnew == 0) && (slice_good1 == 0) && (slice_good2 == 0) &&(slice_good3 == 0) &&(slice_good7 == 0) &&(slice_good8 == 0) && (slice_good9 == 0) && (slice_good10 == 0)) { 		
	braille_cell = slice_good5 *8; 				//if assume is left side print  
	printf("  braille cell = %d", braille_cell);
	braille_cell = slice_good5;					//if assume is right side
	printf("@%d", braille_cell);}

slice_good10 = slice_good9;		//refresh the reference correct slices
slice_good9 = slice_good8;
slice_good8 = slice_good7;		
slice_good7 = slice_good6;
slice_good6 = slice_good5;
slice_good5 = slice_good4;	
slice_good4 = slice_good3;	
slice_good3 = slice_good2;
slice_good2 = slice_good1;
slice_good1 = slice_goodnew;

i = 0;				//re-initialise after each interrupt, for sample array
brd_led_toggle(BRD_LED2 );


//-------------------------------4. determine braille cell---------------------------------
/*-----------------------------5. send to host(use rtdx, does not require dsp/bios)-------*/
asm(" STM #5,1h");	// Clear any previous interrupt in IFR
asm(" STM #1,0h");		// Enable the int0 interrupt, disable SI(int1) in IMR
asm(" RSBX INTM");		// globally disable interrupts		
}
