/*********************************************************************************
CCD Output - braille project C5402 target software
Andrew Paquale 09913770
26/7/2002
**********************************************************************************/

#include <type.h>
#include <board.h>
#include <codec.h>
#include <mcbsp54.h>
#include <stdio.h>
#include "target.h"
#include "intr.h"


void codewheel_interrupt0();
void SI_interrupt2();

/*****************************************************************************/
/* Global Variables                                                          */
/*****************************************************************************/

HANDLE hHandset;
s16 data;
int data_array[200];
static short codewheel_var = 0;
int i = 0, newline_var = 0;

typedef unsigned int	word;
//#define		SWWSR	(*((volatile word*)0x0028))


extern int bio_test()
{
asm("_test_bio:");
asm("	LD #0, A ;Set return value to false ");
asm("	BC bio_low, NBIO ;NBIO = high");
asm("	;BIO = 0 => moving forward");
asm("	NOP");
asm("	NOP ");
asm("	LD #1, A ;Set return value to true ");
asm("	RET ;Card present, return normally");
asm("bio_low: ;bio is low ");
asm("	RET ;exit");
}

void setup_interrupt()
{
	asm(" SSBX INTM");	// Globally disable interrupts
	asm(" STM #5,1h");	// Clear any previous interrupt in IFR
	asm(" STM #1,0h");	// Enable the int0 interrupt, disable SI(int1) in IMR
	asm(" RSBX INTM");	// Globally enable interrupts
}
/*****************************************************************************************
--------------------------------------- main function ------------------------------------
1. set up interrupts
2. set up tlc320AD50 (initailise MsCbs serial port, set sampling rate etc)
3. wait for events
*****************************************************************************************/

void main()
{
    
    if (brd_init(100))
        return;

INTR_INIT
INTR_HOOK( 16,codewheel_interrupt0)
INTR_HOOK( 18,SI_interrupt2)

//-------------------------1. set up interrupts-------------------------------------------	
	// Setup the interrupt handler
	setup_interrupt();
	port0 = 0x80;			//sets CNTL1 register, enable int2
	
	
    /* Open Handset Codec */
    hHandset = codec_open(HANDSET_CODEC);               // Acquire handle to codec 

    // Set codec parameters 
    codec_dac_mode(hHandset, CODEC_DAC_15BIT);          // DAC in 15-bit mode 
    codec_adc_mode(hHandset, CODEC_ADC_15BIT);          // ADC in 15-bit mod
    codec_ain_gain(hHandset, CODEC_AIN_6dB);            // 6dB gain on analog input to ADC 
    codec_aout_gain(hHandset, CODEC_AOUT_MINUS_0dB);    // -6dB gain on analog output from DAC 
    codec_sample_rate(hHandset,SR_16000);               // 16KHz sampling rate 


while (1);
    /* Polling and digital loopback */
    while (i < 200)
    {
       /* Wait for sample from handset */
       while (!MCBSP_RRDY(HANDSET_CODEC)) {};

       /* Read sample from and write back to handset codec */
       data = *(volatile u16*)DRR1_ADDR(HANDSET_CODEC);
       *(volatile u16*)DXR1_ADDR(HANDSET_CODEC) = data;
       data_array[i] = data;
	   ++i;
    }
 i = 1;
 while (i < 200) {
 	if (newline_var == 20) {printf("\n"); newline_var = 0;}
	if (data_array[i] > 1000) { printf(" %d",  data_array[i]);}
	++i;
	++newline_var;
	}
	return;
}


/**************************************************************************************
---------------------- codewheel_interrupt --------------------------------------------
The SI Pulse provides an interrupt to X_INT1 every 5.3 mS, 
the DSP will only process the CCD image slice from the AIC if 
a X_INT0 event (indicating a codewheel trigger signal) 
occurred in the last 5.3 mS AND XF equals zero (indicating forward motion)
***************************************************************************************/
interrupt void codewheel_interrupt0()
{
asm(" SSBX INTM");		// Disable interrupts


codewheel_var = bio_test();		//test to see if codewheel is moving forward
if (codewheel_var == 1) { 		//enable the SI interrupt
	brd_led_enable(BRD_LED0); 
	brd_led_enable(BRD_LED1);
	asm("	SSBX 1, 13");
	asm("	STM #1h,IFR");	// Clear the interrupt0 so may be used again (IFR)
	asm(" 	STM #4h,0h");		//but disable it until SI interrupt service routine 
	}							//completes,and enable SI interrupt 1 (IMR)
	
if (codewheel_var == 0)	{ 		//keep waiting, dont enable SI interrupt
	brd_led_disable(BRD_LED0); 
	brd_led_disable(BRD_LED1);
	brd_led_disable(BRD_LED2);
	}
asm(" STM #5,1h");	// Clear any previous interrupt in IFR
asm(" STM #4,0h");	// Enable the int2 interrupt, disable codewheel(int0) in IMR
asm(" RSBX INTM");		//globally enable interrupts

}

/************************************************************************************
-------------------- SI_interrupt ---------------------------------------------------
1. record the CCD output (57 samples after SI pulse)
2. determine dots within image slice of 57 samples
3. store then correlate with previous or latter slices constituting a braille cell
4. determine braille cell
5. send to host(use rtdx, does not require dsp/bios)
*************************************************************************************/
interrupt void SI_interrupt2()
{

asm(" SSBX INTM");		// Disable interrupts

brd_led_enable(BRD_LED2);



//----------------------- 1. record the CCD output (57 samples after SI pulse)--------------	

/*--------------------2. determine dots within image slice of 57 samples--------------------
  a. use at least 3 image slices to detemine dots in a "slice', since ccd
 	 may not be perfectly horizontal
  b. for every 5 or so ccd samples (breaking image slice into about 10 parts), check
  	 to see if a dot exists. Since the distance between dots is precisely known and
  	 constant from the first dot found, corresponding area's where the dots could be 
  	 found are checked. Therefore the actual place of the first dot can vary and proper
  	 braille cell type can still be determined*/

/*-----3. store then correlate with previous or latter slices constituting a braille cell---
	- Since initially the ccd could be placed anywhere on the braille line, the first image 
	slice will only be used if the next image slice contituting dots is inside the width
	of a braille cell (determined by counting the codewheel interrupts) otherwise it is
	discarded (as being the the second row of a braille cell)
	*/
//-------------------------------4. determine braille cell---------------------------------
/*-----------------------------5. send to host(use rtdx, does not require dsp/bios)-------*/
asm(" STM #5,1h");	// Clear any previous interrupt in IFR
asm(" STM #1,0h");		// Enable the int0 interrupt, disable SI(int1) in IMR
asm(" RSBX INTM");		// globally enable interrupts		
}
