/*********************************************************************************
CCD Output - braille project C5402 target software
Andrew Paquale 09913770
26/7/2002
**********************************************************************************/

#include <type.h>
#include <board.h>
#include <codec.h>
#include <mcbsp54.h>
#include <stdio.h>
#include "target.h"
#include "intr.h"

void codewheel_interrupt0();
void SI_interrupt2();

/*****************************************************************************/
/* Global Variables                                                          */
/*****************************************************************************/
HANDLE hHandset;
s16 data;
int data_array[60];
static short codewheel_var = 0;
int i = 0, newline_var = 0;
int reference_amplitude;		
char temp[80];

extern int bio_test()
{
asm("_test_bio:");
asm("	LD #0, A ;Set return value to false ");
asm("	BC bio_low, NBIO ;NBIO = high");
asm("	;BIO = 0 => moving forward");
asm("	NOP");
asm("	NOP ");
asm("	LD #1, A ;Set return value to true ");
asm("	RET ;Card present, return normally");
asm("bio_low: ;bio is low ");
asm("	RET ;exit");
}

void setup_interrupt()
{
	asm(" SSBX INTM");	// Globally disable interrupts
	asm(" STM #5,1h");	// Clear any previous interrupt in IFR
	asm(" STM #4,0h");	// Enable the int2 interrupt, disable codewheel(int0) in IMR
	asm(" RSBX INTM");	// Globally enable interrupts
}
/*****************************************************************************************
--------------------------------------- main function ------------------------------------
1. set up interrupts
2. set up tlc320AD50 (initailise MsCbs serial port, set sampling rate etc)
3. wait for events
*****************************************************************************************/

void main()
{
    
    if (brd_init(100))
        return;

//-------------------------1. set up interrupts-------------------------------------------	
	
	INTR_INIT								//initailise pointer to vector table
	INTR_HOOK( 16,codewheel_interrupt0)		//put ISR into vector table
	INTR_HOOK( 18,SI_interrupt2)			//put ISR into vector table
	
	// Setup the interrupt handler
	setup_interrupt();
	port0 = 0x80;			//sets CNTL1 register, enable int2
	
//-------------------------2. Setup the ADC ---------------------------------------------	
	
    /* Open Handset Codec */
    hHandset = codec_open(HANDSET_CODEC);               // Acquire handle to codec 

    // Set codec parameters 
    codec_adc_mode(hHandset, CODEC_ADC_15BIT);          // ADC in 15-bit mod
    codec_ain_gain(hHandset, CODEC_AIN_0dB);            // 0dB gain on analog input to ADC 
    codec_sample_rate(hHandset,SR_16000);               // 16KHz sampling rate 
	i = 0;

while (1);
    /* Polling and digital loopback */
    
	return;
}


/**************************************************************************************
---------------------- codewheel_interrupt --------------------------------------------
The SI Pulse provides an interrupt to X_INT1 every 5.3 mS, 
the DSP will only process the CCD image slice from the AIC if 
a X_INT0 event (indicating a codewheel trigger signal) 
occurred in the last 5.3 mS AND XF equals zero (indicating forward motion)
***************************************************************************************/
interrupt void codewheel_interrupt0()
{
asm(" SSBX INTM");		// Disable interrupts


codewheel_var = bio_test();		//test to see if codewheel is moving forward
if (codewheel_var == 1) { 		//enable the SI interrupt
	brd_led_enable(BRD_LED0); 
	brd_led_enable(BRD_LED1);
	asm("	SSBX 1, 13");
	asm("	STM #1h,IFR");	// Clear the interrupt0 so may be used again (IFR)
	asm(" 	STM #4h,0h");		//but disable it until SI interrupt service routine 
	}							//completes,and enable SI interrupt 1 (IMR)
	
if (codewheel_var == 0)	{ 		//keep waiting, dont enable SI interrupt
	brd_led_disable(BRD_LED0); 
	brd_led_disable(BRD_LED1);
	brd_led_disable(BRD_LED2);
	}
asm(" STM #5,1h");	// Clear any previous interrupt in IFR
asm(" STM #4,0h");	// Enable the int2 interrupt, disable codewheel(int0) in IMR
asm(" RSBX INTM");		//globally enable interrupts

}

/************************************************************************************
-------------------- SI_interrupt ---------------------------------------------------
1. record the CCD output (57 samples after SI pulse)
2. determine dots within image slice of 57 samples
3. store then correlate with previous or latter slices constituting a braille cell
4. determine braille cell
5. send to host(use rtdx, does not require dsp/bios)
*************************************************************************************/
interrupt void SI_interrupt2()
{

asm(" SSBX INTM");		// Disable interrupts

//----------------------- 1. record the CCD output (57 samples after SI pulse)--------------	
while (i < 59)
    {
       /* Wait for sample from handset */
       while (!MCBSP_RRDY(HANDSET_CODEC)) {};

       /* Read sample from and write back to handset codec */
       data = *(volatile u16*)DRR1_ADDR(HANDSET_CODEC);
       data_array[i] = data;
	   ++i;
    }

//--------------------2. determine dots within image slice of 57 samples--------------------
 	
reference_amplitude = ((data_array[28] / 3) + (data_array[29] / 3) + (data_array[30] / 3));	//divide by 3 to prevent overflow
i = 52;
while (i > 27) {
	if (data_array[i] > (reference_amplitude / 8)) {	printf("1");}
	else {printf (" ");}
	--i; 
	}
printf("\n");		//new line for each slice
i = 0;				//re-initialise after each interrupt


/*//to determine if a dot 1 exists (by shadow)
if ((reference_amplitude - data_array[27]) > 1000) { sam27 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[28]) > 1000) { sam28 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[29]) > 1000) { sam29 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[30]) > 1000) { sam30 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[31]) > 1000) { sam31 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[32]) > 1000) { sam32 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[33]) > 1000) { sam33 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[34]) > 1000) { sam34 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[35]) > 1000) { sam35 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[36]) > 1000) { sam36 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[37]) > 1000) { sam37 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[38]) > 1000) { sam38 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[39]) > 1000) { sam39 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[40]) > 1000) { sam40 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[41]) > 1000) { sam41 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[42]) > 1000) { sam42 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[43]) > 1000) { sam43 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[44]) > 1000) { sam44 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[45]) > 1000) { sam45 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[46]) > 1000) { sam46 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[47]) > 1000) { sam47 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[48]) > 1000) { sam48 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[49]) > 1000) { sam49 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[50]) > 1000) { sam50 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[51]) > 1000) { sam51 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[52]) > 1000) { sam52 = 1;}  else { sam = 0;}
if ((reference_amplitude - data_array[53]) > 1000) { sam53 = 1;}  else { sam = 0;}
*/


brd_led_toggle(BRD_LED1);

/*-----------------------------5. send to host(use rtdx, does not require dsp/bios)-------*/
asm(" STM #5,1h");	// Clear any previous interrupt in IFR
asm(" STM #1,0h");		// Enable the int0 interrupt, disable SI(int1) in IMR
asm(" RSBX INTM");		// globally disable interrupts		
}
