/*
 *  Copyright 2001 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *  
 */
/* "@(#) DSP/BIOS 4.60.22 12-07-01 (barracuda-j15)" */
/******************************************************************************\
*           Copyright (C) 1999 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* MODULE NAME... CSL - chip support library service layer interface
* FILENAME...... csl.h
* DATE CREATED.. Fri 06/11/1999 
* PROJECT....... Chip Support Library
* COMPONENT..... HAL
* PREREQUISITS.. 
*------------------------------------------------------------------------------
* HISTORY:
*   CREATED:  06/11/1999 Created
*   MODIFIED: 11/13/1999 Modified for TMS320C54xx
*   MODIFIED: 03/21/2001 Fixed bug in _CSL_init (assign to _CSL_version
*                        before call to CSL_init() 
*   MODIFIED: 06/10/2001 Modified for new initialziation model
*   MODIFIED: 07/09/2001 added support for 5401, 5471, removed support for 5472
*------------------------------------------------------------------------------
* DESCRIPTION:  (HAL interface file for the CSL library)
*
*
*
\******************************************************************************/
#ifndef _CSL_H_
#define _CSL_H_

#include <csl_stdinc.h>
#include <csl_stdinch.h>
#include <csl_err.h>
#include <csl_csldat.h>
#include <csl_chiphal.h>
#include <csl_timer.h>
#include <csl_irq.h>

/****************************************\
* CSL scope and inline control macros
\****************************************/
#ifdef __cplusplus
#define CSLAPI extern "C"  
#else
#define CSLAPI extern 
#endif

#undef  USEDEFS
#undef  IDECL
#undef  IDEF

#ifdef  _CSL_MOD_
  #define IDECL CSLAPI
  #define IDEF
#else
  #ifdef  _INLINE
    #define IDECL static inline
    #define USEDEFS
    #define IDEF  static inline
  #else
    #define IDECL CSLAPI
  #endif
#endif

/****************************************\
* CSL global macro declarations
\****************************************/

/****************************************\
* CSL global typedef declarations
\****************************************/
typedef union {
  struct {
    Uint16 biosPresent;
    _IRQ_Dispatch *dispatchTable;
    Uint16 timerUsed;
    Uint16 timerNum;
  } args;
  struct {
    TIMER_Handle hTimer;
    Uint16 *event2IntTbl;
    Uint16 *int2EventTbl;
  } ret;
} _CSL_Config;

/****************************************\
* CSL global variable declarations
\****************************************/
#define __CSL_VERSION         "CSL VERSION 1.10 ["__DATE__"] ["__TIME__"]";
#define __CSL_DATESTAMP       "["__DATE__"] ["__TIME__"]";


/****************************************\
* CSL global function declarations
\****************************************/
CSLAPI void CSL5401_LIB_();
CSLAPI void CSL5402_LIB_();
CSLAPI void CSL5409_LIB_();
CSLAPI void CSL5409A_LIB_();
CSLAPI void CSL5410_LIB_();
CSLAPI void CSL5410A_LIB_();
CSLAPI void CSL5416_LIB_();
CSLAPI void CSL5420_LIB_();
CSLAPI void CSL5421_LIB_();
CSLAPI void CSL5440_LIB_();
CSLAPI void CSL5441_LIB_();
CSLAPI void CSL5471_LIB_();
CSLAPI void CSLALL_LIB_();


/****************************************\
* CSL inline function declarations
\****************************************/

IDECL void CSL_init(void);
IDECL void _CSL_init(_CSL_Config *config);

/*******************************************************************************\
* Chip Support Library Global Initialization Function
\*******************************************************************************/

#ifdef USEDEFS
  IDEF void CSL_init(void) {

   *(_CSL_SysStatusPtr*)CSL_DATA_PTR = &_CSL_DataObj;

   CSL_SYS_DATA.DmaAllocMask        = 0x0000u;  //Initialize Resource Usage Masks
   CSL_SYS_DATA.DmaGblAllocMask     = 0x0000u;
   CSL_SYS_DATA.McbspPortAllocMask  = 0x0000u;
   CSL_SYS_DATA.TimerAllocMask      = 0x0000u;
   CSL_SYS_DATA.PllAllocMask        = 0x0000u;
   CSL_SYS_DATA.WdtimAllocMask      = 0x0000u;
   CSL_SYS_DATA.DatAllocMask        = 0x0000u;

   ERR_init();

  #if (CHIP_5401)
    CSL5401_LIB_();
  #elif (CHIP_5402)
    CSL5402_LIB_();
  #elif (CHIP_5409)
    CSL5409_LIB_();
  #elif (CHIP_5409A)
    CSL5409A_LIB_();
  #elif (CHIP_5410)
    CSL5410_LIB_();
  #elif (CHIP_5410A)
    CSL5410A_LIB_();
  #elif (CHIP_5416)
    CSL5416_LIB_();
  #elif (CHIP_5420)
    CSL5420_LIB_();
  #elif (CHIP_5421)
    CSL5421_LIB_();
  #elif (CHIP_5440)
    CSL5440_LIB_();
  #elif (CHIP_5441)
    CSL5441_LIB_();
  #elif (CHIP_5471)
    CSL5471_LIB_();
  #elif (CHIP_ALL)
    CSLALL_LIB_();
  #endif

  }  
 /*-----------------------------------------------------------------------------*/
void _CSL_init(_CSL_Config *config) {

  Uint16 old_intm;
  TIMER_Handle hTimer   = (TIMER_Handle)INV;
  Uint16 biosPresent    = config->args.biosPresent;
  _IRQ_Dispatch *dispatchTable = config->args.dispatchTable;
  Uint16 timerUsed      = config->args.timerUsed;
  Uint16 timerNum       = config->args.timerNum;



  CSL_init();

  CSL_SYS_DATA.Version = __CSL_VERSION;
  CSL_SYS_DATA.DateStamp = __CSL_VERSION;


  old_intm = IRQ_globalDisable();

  _IRQ_init(biosPresent, dispatchTable);

  if (timerUsed) {
    hTimer = TIMER_open(timerNum,0); 
  }

  config->ret.hTimer = hTimer;
  config->ret.event2IntTbl = CSL_IRQ_DATA.IrqEventTable;
  config->ret.int2EventTbl = CSL_IRQ_DATA.IrqIntTable;

  IRQ_globalRestore(old_intm);
}

#endif /* USEDEFS */ 
#endif /* _CSL_H_ */
/******************************************************************************\
* End of csl.h
\******************************************************************************/

